// DEMO.HPP
#ifndef __DEMO_HPP
#define __DEMO_HPP
#include "dos640k.hpp"
#include "3dmath.hpp"
#include "xlib_all.h"
#include "xtended.cpp" //


typedef struct {
	BYTE r,g,b;
} RGB_t;

typedef struct  {
	RGB_t col[256];
} PaletteRaw_t;

typedef struct {
	BYTE start_color, num_colors;
	PaletteRaw_t pal_raw;
} PaletteStruct_t;

class Palette {
	public:
		//PaletteStruct_t data;
		BYTE *PalRaw, *PalStruct;
	public:
		Palette();
		BYTE* operator[](BYTE);
		void Set();
		void GetCurrent();
		void FadeScreenTo();
		boolean FadeTowards(Palette);
};
Palette::Palette() {
	PalStruct = new BYTE[256*3+2];
	PalRaw = PalStruct+2;
}
void Palette::Set(){
	x_put_pal_struc(PalStruct);
}
void Palette::GetCurrent(){
	x_get_pal_struc(PalStruct,255,0);
}
BYTE* Palette::operator[](BYTE ind){  //Palette[index][r(0)/g(1)/b(2)]
	return  PalRaw+(ind+ind+ind);
}
boolean Palette::FadeTowards(Palette pal2){
	int j, diff;
	boolean done=TRUE;
	for(j=0;j<256*3;j++)
	{
			diff = PalRaw[j]-pal2.PalRaw[j];
			if(diff>0){
				--PalRaw[j];
					if(diff>8) --PalRaw[j];
				done=FALSE;
			}else
				if(diff<0){
					++PalRaw[j];
					if(diff<-8) ++PalRaw[j];
				done=FALSE;
			};
	}
	return done;
}


void Palette::FadeScreenTo(){
	Palette pal;
	pal.GetCurrent();
	boolean done=FALSE;
	int j, diff;
	do{
		done=pal.FadeTowards(*this);
		pal.Set(); //x_put_pal_struc(pal.PalStruct);
		x_wait_vsync();
	}while(!done);
}


Palette OriginalPalette;

//Graphic stuff .... VERY unstructured... XLIB/DEMO-specific
/*unsigned int PageScr[2];
unsigned int& LogicalScr = PageScr[1];
unsigned int& PhysicalScr = PageScr[2];
*/
void PageFlip(){
	x_page_flip(0,0);
	unsigned int tmp = PageScr[0];
	PageScr[0]=PageScr[1];
	PageScr[1]=tmp;
}

enum Demo_Graphic_Modes { DGM_Unknown=0, DGM_TextMode,
		 DGM_Mode13h, DGM_ModeX, DGM_Xtended };
Demo_Graphic_Modes DemoGraphicMode = DGM_Unknown;
boolean SetDemoGraphicMode(Demo_Graphic_Modes gm){
	boolean res=TRUE;
	switch (gm) {
		case DGM_Unknown	: res=FALSE; break;
		case DGM_TextMode	:	x_text_mode(); break;
		case DGM_Mode13h	:	enter_mode_13h(); break;
		case DGM_ModeX 		:
				x_set_mode(X_MODE_320x240,320); //320x400 max res, w/2 pages(256kB limit)
				x_set_doublebuffer(ScrnPhysicalHeight);
				PageScr[0] = Page0_Offs;		 PageScr[1] = Page1_Offs;
			break;
		//case DGM_Xtended	: Init_Xtended(); break;
		}
		if(res) DemoGraphicMode=gm;
	return res;
}







#endif
